"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[39749],{84500:(a,t,e)=>{e.r(t),e.d(t,{icon:()=>i});e(65043);var r=e(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(a){for(var t=1;t<arguments.length;t++){var e=arguments[t];for(var r in e)({}).hasOwnProperty.call(e,r)&&(a[r]=e[r])}return a},l.apply(null,arguments)}var i=function(a){var t=a.title,e=a.titleId,i=function(a,t){if(null==a)return{};var e,r,n=function(a,t){if(null==a)return{};var e={};for(var r in a)if({}.hasOwnProperty.call(a,r)){if(t.indexOf(r)>=0)continue;e[r]=a[r]}return e}(a,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(a);for(r=0;r<l.length;r++)e=l[r],t.indexOf(e)>=0||{}.propertyIsEnumerable.call(a,e)&&(n[e]=a[e])}return n}(a,n);return(0,r.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":e},i),t?(0,r.Y)("title",{id:e},t):null,(0,r.Y)("path",{d:"M2 2.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1H3v10h1.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5v-11Z"}),(0,r.Y)("path",{d:"M6 5a1 1 0 1 1-2 0 1 1 0 0 1 2 0Zm0 3a1 1 0 1 1-2 0 1 1 0 0 1 2 0Zm-1 4a1 1 0 1 0 0-2 1 1 0 0 0 0 2Zm4-7a1 1 0 1 1-2 0 1 1 0 0 1 2 0ZM8 9a1 1 0 1 0 0-2 1 1 0 0 0 0 2Zm1 2a1 1 0 1 1-2 0 1 1 0 0 1 2 0Zm2-5a1 1 0 1 0 0-2 1 1 0 0 0 0 2Zm1 2a1 1 0 1 1-2 0 1 1 0 0 1 2 0Zm-1 4a1 1 0 1 0 0-2 1 1 0 0 0 0 2Z"}),(0,r.Y)("path",{d:"M13.5 2a.5.5 0 0 1 .5.5v11a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1H13V3h-1.5a.5.5 0 0 1 0-1h2Z"}))}}}]);
//# sourceMappingURL=icon.tokenVectorDense.6e8dbc4b.chunk.js.map