"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[67562],{84401:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>i});r(65043);var n=r(77724),l=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,r=e.titleId,i=function(e,t){if(null==e)return{};var r,n,l=function(e,t){if(null==e)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;r[n]=e[n]}return r}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++)r=a[n],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(l[r]=e[r])}return l}(e,l);return(0,n.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":r},i),t?(0,n.Y)("title",{id:r},t):null,(0,n.Y)("path",{d:"M5.394 6.192 11.05.536 16 5.486l4.95-4.95 5.657 5.656L16 16.8 5.394 6.192Zm2.828 0L16 13.971l7.778-7.779-2.828-2.828L16 8.314l-4.95-4.95-2.828 2.828Z",className:"euiIcon__fillSecondary"}),(0,n.Y)("path",{d:"M16 30.941 4.686 19.627A8.004 8.004 0 0 1 3.45 9.905a8.011 8.011 0 0 1 1.236-1.591L16 19.539 27.313 8.314l.045.045a8.003 8.003 0 0 1-.045 11.268L16 30.942Zm0-2.828 9.899-9.9a6.002 6.002 0 0 0 1.169-6.838L16 22.357 4.932 11.375A6.003 6.003 0 0 0 6.1 18.213l9.9 9.9Z"}))}}}]);
//# sourceMappingURL=icon.app_vulnerability_management.3691447a.chunk.js.map