"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[54694],{44117:(t,e,r)=>{r.r(e),r.d(e,{icon:()=>a});r(65043);var l=r(77724),n=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var l in r)({}).hasOwnProperty.call(r,l)&&(t[l]=r[l])}return t},i.apply(null,arguments)}var a=function(t){var e=t.title,r=t.titleId,a=function(t,e){if(null==t)return{};var r,l,n=function(t,e){if(null==t)return{};var r={};for(var l in t)if({}.hasOwnProperty.call(t,l)){if(e.indexOf(l)>=0)continue;r[l]=t[l]}return r}(t,e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(t);for(l=0;l<i.length;l++)r=i[l],e.indexOf(r)>=0||{}.propertyIsEnumerable.call(t,r)&&(n[r]=t[r])}return n}(t,n);return(0,l.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":r},a),e?(0,l.Y)("title",{id:r},e):null,(0,l.Y)("path",{fill:"#00BFB3",d:"M11.05 1.95 6.808 6.192 16 15.385l9.192-9.193L20.95 1.95 16 6.9l-4.95-4.95Z"}),(0,l.Y)("path",{fill:"#FA744E",d:"M4.686 19.627a8 8 0 0 1 0-11.313L16 19.627v11.314L4.686 19.627Z"}),(0,l.Y)("path",{fill:"#343741",d:"M27.314 8.314a8 8 0 0 1 0 11.313L16 30.941V19.627L27.314 8.314Z"}))}}}]);
//# sourceMappingURL=icon.logo_vulnerability_management.d1e02609.chunk.js.map