"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[57994],{7137:(e,l,t)=>{t.r(l),t.d(l,{icon:()=>a});t(65043);var n=t(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var t=arguments[l];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},i.apply(null,arguments)}var a=function(e){var l=e.title,t=e.titleId,a=function(e,l){if(null==e)return{};var t,n,r=function(e,l){if(null==e)return{};var t={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(l.indexOf(n)>=0)continue;t[n]=e[n]}return t}(e,l);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)t=i[n],l.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(r[t]=e[t])}return r}(e,r);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":t},a),l?(0,n.Y)("title",{id:t},l):null,(0,n.Y)("g",{fill:"none",fillRule:"evenodd"},(0,n.Y)("path",{fill:"#FA744E",d:"M27.05 10h-7.34l-11 22s15.696-4.96 21.855-16.076C32.037 13.266 30.088 10 27.05 10"}),(0,n.Y)("path",{fill:"#00BFB3",d:"M21.355 0H7.533L.427 14.211C-.903 16.871 1.032 20 4.004 20h7.351l10-20Z"}),(0,n.Y)("path",{d:"M2.533 10 .428 14.211C-.903 16.871 1.032 20 4.005 20h7.35l5-10H2.533Z",className:"euiIcon__fillNegative"})))}}}]);
//# sourceMappingURL=icon.logo_site_search.e5b2d3d4.chunk.js.map