"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[62837],{84220:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>i});l(65043);var r=l(77724),a=["title","titleId"];function n(){return n=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var r in l)({}).hasOwnProperty.call(l,r)&&(e[r]=l[r])}return e},n.apply(null,arguments)}var i=function(e){var t=e.title,l=e.titleId,i=function(e,t){if(null==e)return{};var l,r,a=function(e,t){if(null==e)return{};var l={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;l[r]=e[r]}return l}(e,t);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);for(r=0;r<n.length;r++)l=n[r],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(a[l]=e[l])}return a}(e,a);return(0,r.Y)("svg",n({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":l},i),t?(0,r.Y)("title",{id:l},t):null,(0,r.Y)("path",{d:"M9.146 10.146a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 0 1 .708-.708L7 11.293V7.5a.5.5 0 0 1 1 0v3.793l1.146-1.147Z"}),(0,r.Y)("path",{fillRule:"evenodd",d:"M15 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h11a2 2 0 0 1 2 2Zm-1 2.707V5h-1.293l-4-4h1.586L14 4.707ZM1 6h13v7a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V6Zm1.293-1H1V3.707L2.293 5Zm3 0H3.707L1 2.293V2a1 1 0 0 1 .455-.838L5.293 5Zm3 0H6.707l-4-4h1.586l4 4Zm3 0H9.707l-4-4h1.586l4 4ZM14 3.293V2a1 1 0 0 0-1-1h-1.293L14 3.293Z"}))}}}]);
//# sourceMappingURL=icon.transitionTopIn.babe5dec.chunk.js.map