# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2
from feast.protos.feast.core import Entity_pb2 as feast_dot_core_dot_Entity__pb2
from feast.protos.feast.core import FeatureService_pb2 as feast_dot_core_dot_FeatureService__pb2
from feast.protos.feast.core import FeatureView_pb2 as feast_dot_core_dot_FeatureView__pb2
from feast.protos.feast.core import InfraObject_pb2 as feast_dot_core_dot_InfraObject__pb2
from feast.protos.feast.core import OnDemandFeatureView_pb2 as feast_dot_core_dot_OnDemandFeatureView__pb2
from feast.protos.feast.core import Permission_pb2 as feast_dot_core_dot_Permission__pb2
from feast.protos.feast.core import Project_pb2 as feast_dot_core_dot_Project__pb2
from feast.protos.feast.core import Registry_pb2 as feast_dot_core_dot_Registry__pb2
from feast.protos.feast.core import SavedDataset_pb2 as feast_dot_core_dot_SavedDataset__pb2
from feast.protos.feast.core import StreamFeatureView_pb2 as feast_dot_core_dot_StreamFeatureView__pb2
from feast.protos.feast.core import ValidationProfile_pb2 as feast_dot_core_dot_ValidationProfile__pb2
from feast.protos.feast.registry import RegistryServer_pb2 as feast_dot_registry_dot_RegistryServer__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class RegistryServerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ApplyEntity = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyEntity',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyEntityRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetEntity = channel.unary_unary(
                '/feast.registry.RegistryServer/GetEntity',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetEntityRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_Entity__pb2.Entity.FromString,
                )
        self.ListEntities = channel.unary_unary(
                '/feast.registry.RegistryServer/ListEntities',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesResponse.FromString,
                )
        self.DeleteEntity = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteEntity',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteEntityRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyDataSource = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyDataSource',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyDataSourceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetDataSource = channel.unary_unary(
                '/feast.registry.RegistryServer/GetDataSource',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetDataSourceRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_DataSource__pb2.DataSource.FromString,
                )
        self.ListDataSources = channel.unary_unary(
                '/feast.registry.RegistryServer/ListDataSources',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesResponse.FromString,
                )
        self.DeleteDataSource = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteDataSource',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteDataSourceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureViewRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.DeleteFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureViewRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetAnyFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetAnyFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewResponse.FromString,
                )
        self.ListAllFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListAllFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsResponse.FromString,
                )
        self.GetFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_FeatureView__pb2.FeatureView.FromString,
                )
        self.ListFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsResponse.FromString,
                )
        self.GetStreamFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetStreamFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetStreamFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_StreamFeatureView__pb2.StreamFeatureView.FromString,
                )
        self.ListStreamFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListStreamFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsResponse.FromString,
                )
        self.GetOnDemandFeatureView = channel.unary_unary(
                '/feast.registry.RegistryServer/GetOnDemandFeatureView',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetOnDemandFeatureViewRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_OnDemandFeatureView__pb2.OnDemandFeatureView.FromString,
                )
        self.ListOnDemandFeatureViews = channel.unary_unary(
                '/feast.registry.RegistryServer/ListOnDemandFeatureViews',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsResponse.FromString,
                )
        self.ApplyFeatureService = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyFeatureService',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureServiceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetFeatureService = channel.unary_unary(
                '/feast.registry.RegistryServer/GetFeatureService',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureServiceRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_FeatureService__pb2.FeatureService.FromString,
                )
        self.ListFeatureServices = channel.unary_unary(
                '/feast.registry.RegistryServer/ListFeatureServices',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesResponse.FromString,
                )
        self.DeleteFeatureService = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteFeatureService',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureServiceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplySavedDataset = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplySavedDataset',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplySavedDatasetRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetSavedDataset = channel.unary_unary(
                '/feast.registry.RegistryServer/GetSavedDataset',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetSavedDatasetRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_SavedDataset__pb2.SavedDataset.FromString,
                )
        self.ListSavedDatasets = channel.unary_unary(
                '/feast.registry.RegistryServer/ListSavedDatasets',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsResponse.FromString,
                )
        self.DeleteSavedDataset = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteSavedDataset',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteSavedDatasetRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyValidationReference = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyValidationReference',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyValidationReferenceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetValidationReference = channel.unary_unary(
                '/feast.registry.RegistryServer/GetValidationReference',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetValidationReferenceRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_ValidationProfile__pb2.ValidationReference.FromString,
                )
        self.ListValidationReferences = channel.unary_unary(
                '/feast.registry.RegistryServer/ListValidationReferences',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesResponse.FromString,
                )
        self.DeleteValidationReference = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteValidationReference',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteValidationReferenceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyPermission = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyPermission',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyPermissionRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetPermission = channel.unary_unary(
                '/feast.registry.RegistryServer/GetPermission',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetPermissionRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_Permission__pb2.Permission.FromString,
                )
        self.ListPermissions = channel.unary_unary(
                '/feast.registry.RegistryServer/ListPermissions',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsResponse.FromString,
                )
        self.DeletePermission = channel.unary_unary(
                '/feast.registry.RegistryServer/DeletePermission',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeletePermissionRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyProject = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyProject',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyProjectRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetProject = channel.unary_unary(
                '/feast.registry.RegistryServer/GetProject',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetProjectRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_Project__pb2.Project.FromString,
                )
        self.ListProjects = channel.unary_unary(
                '/feast.registry.RegistryServer/ListProjects',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsResponse.FromString,
                )
        self.DeleteProject = channel.unary_unary(
                '/feast.registry.RegistryServer/DeleteProject',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteProjectRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ApplyMaterialization = channel.unary_unary(
                '/feast.registry.RegistryServer/ApplyMaterialization',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyMaterializationRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ListProjectMetadata = channel.unary_unary(
                '/feast.registry.RegistryServer/ListProjectMetadata',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataResponse.FromString,
                )
        self.UpdateInfra = channel.unary_unary(
                '/feast.registry.RegistryServer/UpdateInfra',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.UpdateInfraRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetInfra = channel.unary_unary(
                '/feast.registry.RegistryServer/GetInfra',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetInfraRequest.SerializeToString,
                response_deserializer=feast_dot_core_dot_InfraObject__pb2.Infra.FromString,
                )
        self.Commit = channel.unary_unary(
                '/feast.registry.RegistryServer/Commit',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.Refresh = channel.unary_unary(
                '/feast.registry.RegistryServer/Refresh',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.RefreshRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.Proto = channel.unary_unary(
                '/feast.registry.RegistryServer/Proto',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=feast_dot_core_dot_Registry__pb2.Registry.FromString,
                )
        self.GetRegistryLineage = channel.unary_unary(
                '/feast.registry.RegistryServer/GetRegistryLineage',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageResponse.FromString,
                )
        self.GetObjectRelationships = channel.unary_unary(
                '/feast.registry.RegistryServer/GetObjectRelationships',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsResponse.FromString,
                )
        self.ListFeatures = channel.unary_unary(
                '/feast.registry.RegistryServer/ListFeatures',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesResponse.FromString,
                )
        self.GetFeature = channel.unary_unary(
                '/feast.registry.RegistryServer/GetFeature',
                request_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureRequest.SerializeToString,
                response_deserializer=feast_dot_registry_dot_RegistryServer__pb2.Feature.FromString,
                )


class RegistryServerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ApplyEntity(self, request, context):
        """Entity RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetEntity(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListEntities(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteEntity(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyDataSource(self, request, context):
        """DataSource RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDataSource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListDataSources(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteDataSource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyFeatureView(self, request, context):
        """FeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFeatureView(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAnyFeatureView(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListAllFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeatureView(self, request, context):
        """plain FeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStreamFeatureView(self, request, context):
        """StreamFeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListStreamFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetOnDemandFeatureView(self, request, context):
        """OnDemandFeatureView RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListOnDemandFeatureViews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyFeatureService(self, request, context):
        """FeatureService RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeatureService(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatureServices(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFeatureService(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplySavedDataset(self, request, context):
        """SavedDataset RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSavedDataset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSavedDatasets(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSavedDataset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyValidationReference(self, request, context):
        """ValidationReference RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetValidationReference(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListValidationReferences(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteValidationReference(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyPermission(self, request, context):
        """Permission RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPermissions(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeletePermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyProject(self, request, context):
        """Project RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListProjects(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteProject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyMaterialization(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListProjectMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateInfra(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInfra(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Commit(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Refresh(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Proto(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRegistryLineage(self, request, context):
        """Lineage RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetObjectRelationships(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatures(self, request, context):
        """Feature RPCs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeature(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RegistryServerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ApplyEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyEntity,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyEntityRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEntity,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetEntityRequest.FromString,
                    response_serializer=feast_dot_core_dot_Entity__pb2.Entity.SerializeToString,
            ),
            'ListEntities': grpc.unary_unary_rpc_method_handler(
                    servicer.ListEntities,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesResponse.SerializeToString,
            ),
            'DeleteEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteEntity,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteEntityRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyDataSource': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyDataSource,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyDataSourceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetDataSource': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDataSource,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetDataSourceRequest.FromString,
                    response_serializer=feast_dot_core_dot_DataSource__pb2.DataSource.SerializeToString,
            ),
            'ListDataSources': grpc.unary_unary_rpc_method_handler(
                    servicer.ListDataSources,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesResponse.SerializeToString,
            ),
            'DeleteDataSource': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteDataSource,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteDataSourceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureViewRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'DeleteFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureViewRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetAnyFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAnyFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewResponse.SerializeToString,
            ),
            'ListAllFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListAllFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsResponse.SerializeToString,
            ),
            'GetFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureViewRequest.FromString,
                    response_serializer=feast_dot_core_dot_FeatureView__pb2.FeatureView.SerializeToString,
            ),
            'ListFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsResponse.SerializeToString,
            ),
            'GetStreamFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStreamFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetStreamFeatureViewRequest.FromString,
                    response_serializer=feast_dot_core_dot_StreamFeatureView__pb2.StreamFeatureView.SerializeToString,
            ),
            'ListStreamFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListStreamFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsResponse.SerializeToString,
            ),
            'GetOnDemandFeatureView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetOnDemandFeatureView,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetOnDemandFeatureViewRequest.FromString,
                    response_serializer=feast_dot_core_dot_OnDemandFeatureView__pb2.OnDemandFeatureView.SerializeToString,
            ),
            'ListOnDemandFeatureViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListOnDemandFeatureViews,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsResponse.SerializeToString,
            ),
            'ApplyFeatureService': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyFeatureService,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureServiceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetFeatureService': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeatureService,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureServiceRequest.FromString,
                    response_serializer=feast_dot_core_dot_FeatureService__pb2.FeatureService.SerializeToString,
            ),
            'ListFeatureServices': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatureServices,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesResponse.SerializeToString,
            ),
            'DeleteFeatureService': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFeatureService,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureServiceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplySavedDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplySavedDataset,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplySavedDatasetRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetSavedDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSavedDataset,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetSavedDatasetRequest.FromString,
                    response_serializer=feast_dot_core_dot_SavedDataset__pb2.SavedDataset.SerializeToString,
            ),
            'ListSavedDatasets': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSavedDatasets,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsResponse.SerializeToString,
            ),
            'DeleteSavedDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteSavedDataset,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteSavedDatasetRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyValidationReference': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyValidationReference,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyValidationReferenceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetValidationReference': grpc.unary_unary_rpc_method_handler(
                    servicer.GetValidationReference,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetValidationReferenceRequest.FromString,
                    response_serializer=feast_dot_core_dot_ValidationProfile__pb2.ValidationReference.SerializeToString,
            ),
            'ListValidationReferences': grpc.unary_unary_rpc_method_handler(
                    servicer.ListValidationReferences,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesResponse.SerializeToString,
            ),
            'DeleteValidationReference': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteValidationReference,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteValidationReferenceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyPermission,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyPermissionRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPermission,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetPermissionRequest.FromString,
                    response_serializer=feast_dot_core_dot_Permission__pb2.Permission.SerializeToString,
            ),
            'ListPermissions': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPermissions,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsResponse.SerializeToString,
            ),
            'DeletePermission': grpc.unary_unary_rpc_method_handler(
                    servicer.DeletePermission,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeletePermissionRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyProject': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyProject,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyProjectRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetProject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProject,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetProjectRequest.FromString,
                    response_serializer=feast_dot_core_dot_Project__pb2.Project.SerializeToString,
            ),
            'ListProjects': grpc.unary_unary_rpc_method_handler(
                    servicer.ListProjects,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectsResponse.SerializeToString,
            ),
            'DeleteProject': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteProject,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.DeleteProjectRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ApplyMaterialization': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyMaterialization,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ApplyMaterializationRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ListProjectMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.ListProjectMetadata,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataResponse.SerializeToString,
            ),
            'UpdateInfra': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateInfra,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.UpdateInfraRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetInfra': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInfra,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetInfraRequest.FromString,
                    response_serializer=feast_dot_core_dot_InfraObject__pb2.Infra.SerializeToString,
            ),
            'Commit': grpc.unary_unary_rpc_method_handler(
                    servicer.Commit,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'Refresh': grpc.unary_unary_rpc_method_handler(
                    servicer.Refresh,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.RefreshRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'Proto': grpc.unary_unary_rpc_method_handler(
                    servicer.Proto,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=feast_dot_core_dot_Registry__pb2.Registry.SerializeToString,
            ),
            'GetRegistryLineage': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRegistryLineage,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageResponse.SerializeToString,
            ),
            'GetObjectRelationships': grpc.unary_unary_rpc_method_handler(
                    servicer.GetObjectRelationships,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsResponse.SerializeToString,
            ),
            'ListFeatures': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatures,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesResponse.SerializeToString,
            ),
            'GetFeature': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeature,
                    request_deserializer=feast_dot_registry_dot_RegistryServer__pb2.GetFeatureRequest.FromString,
                    response_serializer=feast_dot_registry_dot_RegistryServer__pb2.Feature.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'feast.registry.RegistryServer', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class RegistryServer(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ApplyEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyEntity',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyEntityRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetEntity',
            feast_dot_registry_dot_RegistryServer__pb2.GetEntityRequest.SerializeToString,
            feast_dot_core_dot_Entity__pb2.Entity.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListEntities(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListEntities',
            feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListEntitiesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteEntity',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteEntityRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyDataSource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyDataSource',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyDataSourceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDataSource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetDataSource',
            feast_dot_registry_dot_RegistryServer__pb2.GetDataSourceRequest.SerializeToString,
            feast_dot_core_dot_DataSource__pb2.DataSource.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListDataSources(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListDataSources',
            feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListDataSourcesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteDataSource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteDataSource',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteDataSourceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureViewRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureViewRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAnyFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetAnyFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.GetAnyFeatureViewResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListAllFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListAllFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListAllFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetFeatureViewRequest.SerializeToString,
            feast_dot_core_dot_FeatureView__pb2.FeatureView.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStreamFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetStreamFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetStreamFeatureViewRequest.SerializeToString,
            feast_dot_core_dot_StreamFeatureView__pb2.StreamFeatureView.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListStreamFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListStreamFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListStreamFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetOnDemandFeatureView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetOnDemandFeatureView',
            feast_dot_registry_dot_RegistryServer__pb2.GetOnDemandFeatureViewRequest.SerializeToString,
            feast_dot_core_dot_OnDemandFeatureView__pb2.OnDemandFeatureView.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListOnDemandFeatureViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListOnDemandFeatureViews',
            feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListOnDemandFeatureViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyFeatureService(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyFeatureService',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyFeatureServiceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeatureService(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetFeatureService',
            feast_dot_registry_dot_RegistryServer__pb2.GetFeatureServiceRequest.SerializeToString,
            feast_dot_core_dot_FeatureService__pb2.FeatureService.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatureServices(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListFeatureServices',
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListFeatureServicesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteFeatureService(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteFeatureService',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteFeatureServiceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplySavedDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplySavedDataset',
            feast_dot_registry_dot_RegistryServer__pb2.ApplySavedDatasetRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSavedDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetSavedDataset',
            feast_dot_registry_dot_RegistryServer__pb2.GetSavedDatasetRequest.SerializeToString,
            feast_dot_core_dot_SavedDataset__pb2.SavedDataset.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListSavedDatasets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListSavedDatasets',
            feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListSavedDatasetsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteSavedDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteSavedDataset',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteSavedDatasetRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyValidationReference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyValidationReference',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyValidationReferenceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetValidationReference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetValidationReference',
            feast_dot_registry_dot_RegistryServer__pb2.GetValidationReferenceRequest.SerializeToString,
            feast_dot_core_dot_ValidationProfile__pb2.ValidationReference.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListValidationReferences(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListValidationReferences',
            feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListValidationReferencesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteValidationReference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteValidationReference',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteValidationReferenceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyPermission',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyPermissionRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetPermission',
            feast_dot_registry_dot_RegistryServer__pb2.GetPermissionRequest.SerializeToString,
            feast_dot_core_dot_Permission__pb2.Permission.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListPermissions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListPermissions',
            feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListPermissionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeletePermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeletePermission',
            feast_dot_registry_dot_RegistryServer__pb2.DeletePermissionRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyProject',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyProjectRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetProject',
            feast_dot_registry_dot_RegistryServer__pb2.GetProjectRequest.SerializeToString,
            feast_dot_core_dot_Project__pb2.Project.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListProjects(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListProjects',
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/DeleteProject',
            feast_dot_registry_dot_RegistryServer__pb2.DeleteProjectRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyMaterialization(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ApplyMaterialization',
            feast_dot_registry_dot_RegistryServer__pb2.ApplyMaterializationRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListProjectMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListProjectMetadata',
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListProjectMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateInfra(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/UpdateInfra',
            feast_dot_registry_dot_RegistryServer__pb2.UpdateInfraRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetInfra(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetInfra',
            feast_dot_registry_dot_RegistryServer__pb2.GetInfraRequest.SerializeToString,
            feast_dot_core_dot_InfraObject__pb2.Infra.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Commit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/Commit',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Refresh(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/Refresh',
            feast_dot_registry_dot_RegistryServer__pb2.RefreshRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Proto(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/Proto',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            feast_dot_core_dot_Registry__pb2.Registry.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRegistryLineage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetRegistryLineage',
            feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.GetRegistryLineageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetObjectRelationships(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetObjectRelationships',
            feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.GetObjectRelationshipsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatures(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/ListFeatures',
            feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.ListFeaturesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeature(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.registry.RegistryServer/GetFeature',
            feast_dot_registry_dot_RegistryServer__pb2.GetFeatureRequest.SerializeToString,
            feast_dot_registry_dot_RegistryServer__pb2.Feature.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
