"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[56116],{59603:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>i});r(65043);var a=r(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var a in r)({}).hasOwnProperty.call(r,a)&&(e[a]=r[a])}return e},l.apply(null,arguments)}var i=function(e){var t=e.title,r=e.titleId,i=function(e,t){if(null==e)return{};var r,a,n=function(e,t){if(null==e)return{};var r={};for(var a in e)if({}.hasOwnProperty.call(e,a)){if(t.indexOf(a)>=0)continue;r[a]=e[a]}return r}(e,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(a=0;a<l.length;a++)r=l[a],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}(e,n);return(0,a.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},i),t?(0,a.Y)("title",{id:r},t):null,(0,a.Y)("path",{d:"M3.854 3.146a.5.5 0 0 0-.708 0l-2 2a.5.5 0 1 0 .708.708L3 4.707V11.5a.5.5 0 0 0 1 0V4.707l1.146 1.147a.5.5 0 1 0 .708-.708l-2-2ZM8 4.5a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5ZM8.5 7a.5.5 0 0 0 0 1h4a.5.5 0 0 0 0-1h-4ZM8 10.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5Z"}))}}}]);
//# sourceMappingURL=icon.sortAscending.c08f27f0.chunk.js.map