"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[40345],{57240:(e,a,t)=>{t.r(a),t.d(a,{icon:()=>i});t(65043);var r=t(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var a=1;a<arguments.length;a++){var t=arguments[a];for(var r in t)({}).hasOwnProperty.call(t,r)&&(e[r]=t[r])}return e},l.apply(null,arguments)}var i=function(e){var a=e.title,t=e.titleId,i=function(e,a){if(null==e)return{};var t,r,n=function(e,a){if(null==e)return{};var t={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(a.indexOf(r)>=0)continue;t[r]=e[r]}return t}(e,a);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(r=0;r<l.length;r++)t=l[r],a.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(n[t]=e[t])}return n}(e,n);return(0,r.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":t},i),a?(0,r.Y)("title",{id:t},a):null,(0,r.Y)("path",{d:"M20 14h-8a3 3 0 0 1-3-3V3a3 3 0 0 1 3-3h8a3 3 0 0 1 3 3v8a3 3 0 0 1-3 3ZM12 2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1h-8ZM17 18v-2h-2v2H3v6h2v-4h10v4h2v-4h10v4h2v-6z"}),(0,r.Y)("path",{d:"M4 32a3 3 0 1 1 0-6 3 3 0 0 1 0 6zm0-4a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm12 4a3 3 0 1 1 0-6 3 3 0 0 1 0 6zm0-4a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm12 4a3 3 0 1 1 0-6 3 3 0 0 1 0 6zm0-4a1 1 0 1 0 0 2 1 1 0 0 0 0-2zM13 4h6v2h-6zM13 8h6v2h-6z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_index_pattern.511122bb.chunk.js.map