"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class UserDefinedFunctionV2(google.protobuf.message.Message):
    """Serialized representation of python function."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    BODY_FIELD_NUMBER: builtins.int
    BODY_TEXT_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The function name"""
    body: builtins.bytes
    """The python-syntax function body (serialized by dill)"""
    body_text: builtins.str
    """The string representation of the udf"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        body: builtins.bytes = ...,
        body_text: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["body", b"body", "body_text", b"body_text", "name", b"name"]) -> None: ...

global___UserDefinedFunctionV2 = UserDefinedFunctionV2

class FeatureTransformationV2(google.protobuf.message.Message):
    """A feature transformation executed as a user-defined function"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USER_DEFINED_FUNCTION_FIELD_NUMBER: builtins.int
    SUBSTRAIT_TRANSFORMATION_FIELD_NUMBER: builtins.int
    @property
    def user_defined_function(self) -> global___UserDefinedFunctionV2: ...
    @property
    def substrait_transformation(self) -> global___SubstraitTransformationV2: ...
    def __init__(
        self,
        *,
        user_defined_function: global___UserDefinedFunctionV2 | None = ...,
        substrait_transformation: global___SubstraitTransformationV2 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["substrait_transformation", b"substrait_transformation", "transformation", b"transformation", "user_defined_function", b"user_defined_function"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["substrait_transformation", b"substrait_transformation", "transformation", b"transformation", "user_defined_function", b"user_defined_function"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["transformation", b"transformation"]) -> typing_extensions.Literal["user_defined_function", "substrait_transformation"] | None: ...

global___FeatureTransformationV2 = FeatureTransformationV2

class SubstraitTransformationV2(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUBSTRAIT_PLAN_FIELD_NUMBER: builtins.int
    IBIS_FUNCTION_FIELD_NUMBER: builtins.int
    substrait_plan: builtins.bytes
    ibis_function: builtins.bytes
    def __init__(
        self,
        *,
        substrait_plan: builtins.bytes = ...,
        ibis_function: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ibis_function", b"ibis_function", "substrait_plan", b"substrait_plan"]) -> None: ...

global___SubstraitTransformationV2 = SubstraitTransformationV2
