"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[9416],{96847:(e,t,a)=>{a.r(t),a.d(t,{icon:()=>i});a(65043);var n=a(77724),r=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var a=arguments[t];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},l.apply(null,arguments)}var i=function(e){var t=e.title,a=e.titleId,i=function(e,t){if(null==e)return{};var a,n,r=function(e,t){if(null==e)return{};var a={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;a[n]=e[n]}return a}(e,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(n=0;n<l.length;n++)a=l[n],t.indexOf(a)>=0||{}.propertyIsEnumerable.call(e,a)&&(r[a]=e[a])}return r}(e,r);return(0,n.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":a},i),t?(0,n.Y)("title",{id:a},t):null,(0,n.Y)("path",{d:"M2.909 26.182h1.939v4.848H2.909z",className:"euiIcon__fillSecondary"}),(0,n.Y)("path",{d:"M4.848 16.62V0H2.91v16.62a3.879 3.879 0 1 0 1.94 0Zm-.97 5.683a1.94 1.94 0 1 1 0-3.879 1.94 1.94 0 0 1 0 3.879Z"}),(0,n.Y)("path",{d:"M14.545 16.485h1.939V31.03h-1.939z",className:"euiIcon__fillSecondary"}),(0,n.Y)("path",{d:"M16.485 6.924V0h-1.94v6.924a3.879 3.879 0 1 0 1.94 0Zm-.97 5.682a1.94 1.94 0 1 1 0-3.879 1.94 1.94 0 0 1 0 3.88Z"}),(0,n.Y)("path",{d:"M26.182 26.182h1.939v4.848h-1.939z",className:"euiIcon__fillSecondary"}),(0,n.Y)("path",{d:"M28.121 16.62V0h-1.94v16.62a3.879 3.879 0 1 0 1.94 0Zm-.97 5.683a1.94 1.94 0 1 1 0-3.879 1.94 1.94 0 0 1 0 3.879Z"}))}}}]);
//# sourceMappingURL=icon.app_advanced_settings.3a59f17e.chunk.js.map