"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[9775],{35926:(t,e,r)=>{r.r(e),r.d(e,{icon:()=>a});r(65043);var n=r(77724),l=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var n in r)({}).hasOwnProperty.call(r,n)&&(t[n]=r[n])}return t},i.apply(null,arguments)}var a=function(t){var e=t.title,r=t.titleId,a=function(t,e){if(null==t)return{};var r,n,l=function(t,e){if(null==t)return{};var r={};for(var n in t)if({}.hasOwnProperty.call(t,n)){if(e.indexOf(n)>=0)continue;r[n]=t[n]}return r}(t,e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(t);for(n=0;n<i.length;n++)r=i[n],e.indexOf(r)>=0||{}.propertyIsEnumerable.call(t,r)&&(l[r]=t[r])}return l}(t,l);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},a),e?(0,n.Y)("title",{id:r},e):null,(0,n.Y)("path",{d:"M7 10h2V6H7zM3 10h2V6H3zM11.025 10h2V6h-2zM3.5 13.75h1v-2.4h-1zM6.175 13.75h1.001v-2.4H6.175zM8.85 13.75h1v-2.4h-1zM11.525 13.75h1v-2.4h-1z"}),(0,n.Y)("path",{d:"M0 3v7.05h1v3.698h1v-3.699h12v3.699h1v-3.699h1V3H0Zm1 6h14V4H1v5Z"}))}}}]);
//# sourceMappingURL=icon.memory.fb1802f2.chunk.js.map