"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[19420],{17435:(t,e,a)=>{a.r(e),a.d(e,{icon:()=>i});a(65043);var r=a(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var a=arguments[e];for(var r in a)({}).hasOwnProperty.call(a,r)&&(t[r]=a[r])}return t},l.apply(null,arguments)}var i=function(t){var e=t.title,a=t.titleId,i=function(t,e){if(null==t)return{};var a,r,n=function(t,e){if(null==t)return{};var a={};for(var r in t)if({}.hasOwnProperty.call(t,r)){if(e.indexOf(r)>=0)continue;a[r]=t[r]}return a}(t,e);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(t);for(r=0;r<l.length;r++)a=l[r],e.indexOf(a)>=0||{}.propertyIsEnumerable.call(t,a)&&(n[a]=t[a])}return n}(t,n);return(0,r.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":a},i),e?(0,r.Y)("title",{id:a},e):null,(0,r.Y)("path",{d:"M2 2.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1H3v10h1.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5v-11ZM8 6a1 1 0 1 0 0-2 1 1 0 0 0 0 2Zm0 6a1 1 0 1 0 0-2 1 1 0 0 0 0 2Z"}),(0,r.Y)("path",{d:"M6 8a1 1 0 1 1-2 0 1 1 0 0 1 2 0Zm5 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2Z"}),(0,r.Y)("path",{d:"M13.5 2a.5.5 0 0 1 .5.5v11a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1H13V3h-1.5a.5.5 0 0 1 0-1h2Z"}))}}}]);
//# sourceMappingURL=icon.tokenVectorSparse.022d7602.chunk.js.map