"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[80838],{19821:(a,e,t)=>{t.r(e),t.d(e,{icon:()=>i});t(65043);var n=t(77724),r=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var n in t)({}).hasOwnProperty.call(t,n)&&(a[n]=t[n])}return a},l.apply(null,arguments)}var i=function(a){var e=a.title,t=a.titleId,i=function(a,e){if(null==a)return{};var t,n,r=function(a,e){if(null==a)return{};var t={};for(var n in a)if({}.hasOwnProperty.call(a,n)){if(e.indexOf(n)>=0)continue;t[n]=a[n]}return t}(a,e);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(a);for(n=0;n<l.length;n++)t=l[n],e.indexOf(t)>=0||{}.propertyIsEnumerable.call(a,t)&&(r[t]=a[t])}return r}(a,r);return(0,n.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":t},i),e?(0,n.Y)("title",{id:t},e):null,(0,n.Y)("path",{d:"M29 9H3a3 3 0 0 1-3-3V3a3 3 0 0 1 3-3h26a3 3 0 0 1 3 3v3a3 3 0 0 1-3 3ZM3 2a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h26a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H3Z"}),(0,n.Y)("path",{d:"M12 20H3a3 3 0 0 1-3-3v-3a3 3 0 0 1 3-3h9a3 3 0 0 1 3 3v3a3 3 0 0 1-3 3Zm-9-7a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h9a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1H3Z",className:"euiIcon__fillSecondary"}),(0,n.Y)("path",{d:"M12 31H3a3 3 0 0 1-3-3v-3a3 3 0 0 1 3-3h9a3 3 0 0 1 3 3v3a3 3 0 0 1-3 3Zm-9-7a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h9a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1H3Z"}),(0,n.Y)("path",{d:"M29 31h-9a3 3 0 0 1-3-3V14a3 3 0 0 1 3-3h9a3 3 0 0 1 3 3v14a3 3 0 0 1-3 3Zm-9-18a1 1 0 0 0-1 1v14a1 1 0 0 0 1 1h9a1 1 0 0 0 1-1V14a1 1 0 0 0-1-1h-9Z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_dashboard.512d9586.chunk.js.map