"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[86010],{6609:(l,e,t)=>{t.r(e),t.d(e,{icon:()=>a});t(65043);var r=t(77724),n=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(l){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var r in t)({}).hasOwnProperty.call(t,r)&&(l[r]=t[r])}return l},i.apply(null,arguments)}var a=function(l){var e=l.title,t=l.titleId,a=function(l,e){if(null==l)return{};var t,r,n=function(l,e){if(null==l)return{};var t={};for(var r in l)if({}.hasOwnProperty.call(l,r)){if(e.indexOf(r)>=0)continue;t[r]=l[r]}return t}(l,e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(l);for(r=0;r<i.length;r++)t=i[r],e.indexOf(t)>=0||{}.propertyIsEnumerable.call(l,t)&&(n[t]=l[t])}return n}(l,n);return(0,r.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":t},a),e?(0,r.Y)("title",{id:t},e):null,(0,r.Y)("path",{d:"M7.8 9.812h-.842l-.78 2.335H5.031l.78-2.335H4.6l.182-1.043h1.373l.507-1.504H5.454l.188-1.042h1.367l.792-2.37H8.94l-.792 2.37h.849l.792-2.37h1.145l-.792 2.37H11.4l-.182 1.042H9.8L9.293 8.77h1.248l-.183 1.043H8.946l-.775 2.335H7.026L7.8 9.812Zm-.5-1.043h.842l.513-1.504h-.849L7.3 8.77Z"}))}}}]);
//# sourceMappingURL=icon.tokenNumber.bdb4c068.chunk.js.map