"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[70939],{64410:(e,t,n)=>{n.r(t),n.d(t,{icon:()=>u});n(65043);var r=n(32823),l=n(77724),i=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)({}).hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},a.apply(null,arguments)}var u=function(e){var t=e.title,n=e.titleId,u=function(e,t){if(null==e)return{};var n,r,l=function(e,t){if(null==e)return{};var n={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(r=0;r<i.length;r++)n=i[r],t.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(l[n]=e[n])}return l}(e,i),c=(0,r.Y)("issue");return(0,l.Y)("svg",a({width:16,height:16,viewBox:"0 0 16 16",xmlns:"http://www.w3.org/2000/svg","aria-labelledby":n},u),t?(0,l.Y)("title",{id:n},t):null,(0,l.Y)("g",{clipPath:"url(#".concat(c("a"),")"),fill:"#343741"},(0,l.Y)("path",{fillRule:"evenodd",clipRule:"evenodd",d:"M8 .5c4.136 0 7.5 3.364 7.5 7.5s-3.364 7.5-7.5 7.5S.5 12.136.5 8 3.864.5 8 .5zm0 .882a6.618 6.618 0 100 13.236A6.618 6.618 0 008 1.382z"}),(0,l.Y)("path",{d:"M9 8a1 1 0 11-2 0 1 1 0 012 0z"})),(0,l.Y)("clipPath",{id:c("a")},(0,l.Y)("path",{d:"M0 0h16v16H0z"})))}}}]);
//# sourceMappingURL=icon.issue.06c9d0e5.chunk.js.map