"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[97393],{59224:(t,e,r)=>{r.r(e),r.d(e,{icon:()=>i});r(65043);var l=r(77724),n=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var l in r)({}).hasOwnProperty.call(r,l)&&(t[l]=r[l])}return t},a.apply(null,arguments)}var i=function(t){var e=t.title,r=t.titleId,i=function(t,e){if(null==t)return{};var r,l,n=function(t,e){if(null==t)return{};var r={};for(var l in t)if({}.hasOwnProperty.call(t,l)){if(e.indexOf(l)>=0)continue;r[l]=t[l]}return r}(t,e);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(l=0;l<a.length;l++)r=a[l],e.indexOf(r)>=0||{}.propertyIsEnumerable.call(t,r)&&(n[r]=t[r])}return n}(t,n);return(0,l.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},i),e?(0,l.Y)("title",{id:r},e):null,(0,l.Y)("path",{d:"m8 2 5.196 3v.178l-.866.468V5.5L8 3 3.67 5.5v5L8 13l4.33-2.5V5.77l.866-.474V11L8 14l-5.196-3V5L8 2Z"}),(0,l.Y)("path",{d:"M5.243 4.429 9.597 7.04 8 7.928 3.743 5.563a.5.5 0 1 0-.486.874L7.5 8.794V13.5h1V8.794l4.243-2.357a.508.508 0 0 0 .06-.04l.392-.202V5.047l-.917.505a.573.573 0 0 0-.02.01l-.106.06-.191.105-1.355.753-4.849-2.909-.514.858Z"}))}}}]);
//# sourceMappingURL=icon.tokenModule.edcd8980.chunk.js.map