from feast import Entity

{{ name }} = Entity(
    name="{{ name }}",
    join_keys={{ join_keys }}{% if description %},
    description="{{ description }}"{% endif %}{% if tags %},
    tags={{ tags }}{% endif %}
) 