"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[51346],{73929:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>a});r(65043);var l=r(77724),n=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var l in r)({}).hasOwnProperty.call(r,l)&&(e[l]=r[l])}return e},i.apply(null,arguments)}var a=function(e){var t=e.title,r=e.titleId,a=function(e,t){if(null==e)return{};var r,l,n=function(e,t){if(null==e)return{};var r={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;r[l]=e[l]}return r}(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(l=0;l<i.length;l++)r=i[l],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}(e,n);return(0,l.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":r},a),t?(0,l.Y)("title",{id:r},t):null,(0,l.Y)("path",{d:"M20.38 7.001 17 9.619V0h-2v9.619l-3.37-2.618-1.25 1.513L16 12.878l5.63-4.364z",className:"euiIcon__fillSecondary"}),(0,l.Y)("path",{d:"m21.06.165-1.11 1.61 9.25 5.983L16 16.29 2.8 7.758l9.25-5.983-1.1-1.61L0 7.234v13.653l16 10.337 16-10.337V7.234L21.06.164ZM2 9.57l13 8.407v10.279L2 19.84V9.57Zm15 18.676V17.978l13-8.407V19.85l-13 8.397Z"}))}}}]);
//# sourceMappingURL=icon.app_saved_objects.17462a80.chunk.js.map