"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[48596],{47091:(t,e,l)=>{l.r(e),l.d(e,{icon:()=>i});l(65043);var r=l(77724),a=["title","titleId"];function n(){return n=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var r in l)({}).hasOwnProperty.call(l,r)&&(t[r]=l[r])}return t},n.apply(null,arguments)}var i=function(t){var e=t.title,l=t.titleId,i=function(t,e){if(null==t)return{};var l,r,a=function(t,e){if(null==t)return{};var l={};for(var r in t)if({}.hasOwnProperty.call(t,r)){if(e.indexOf(r)>=0)continue;l[r]=t[r]}return l}(t,e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(t);for(r=0;r<n.length;r++)l=n[r],e.indexOf(l)>=0||{}.propertyIsEnumerable.call(t,l)&&(a[l]=t[l])}return a}(t,a);return(0,r.Y)("svg",n({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":l},i),e?(0,r.Y)("title",{id:l},e):null,(0,r.Y)("path",{d:"M7.276 1.053a.5.5 0 0 1 .448 0l6 3a.5.5 0 0 1 0 .894l-6 3a.5.5 0 0 1-.448 0l-6-3a.5.5 0 0 1 0-.894l6-3zM2.618 4.5 7.5 6.941 12.382 4.5 7.5 2.059 2.618 4.5z"}),(0,r.Y)("path",{d:"M1.053 7.276a.5.5 0 0 1 .67-.223L7.5 9.94l5.776-2.888a.5.5 0 1 1 .448.894l-6 3a.5.5 0 0 1-.448 0l-6-3a.5.5 0 0 1-.223-.67z"}),(0,r.Y)("path",{d:"M1.724 10.053a.5.5 0 1 0-.448.894l6 3a.5.5 0 0 0 .448 0l6-3a.5.5 0 1 0-.448-.894L7.5 12.94l-5.776-2.888z"}))}}}]);
//# sourceMappingURL=icon.layers.c5c75b34.chunk.js.map