"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[99084],{11355:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>i});l(65043);var r=l(77724),n=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var r in l)({}).hasOwnProperty.call(l,r)&&(e[r]=l[r])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,l=e.titleId,i=function(e,t){if(null==e)return{};var l,r,n=function(e,t){if(null==e)return{};var l={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;l[r]=e[r]}return l}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)l=a[r],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(n[l]=e[l])}return n}(e,n);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":l},i),t?(0,r.Y)("title",{id:l},t):null,(0,r.Y)("path",{d:"M1.81 15.19A8.94 8.94 0 0 1 15.62 3.86l.38.42.38-.42a8.94 8.94 0 0 1 14.26 10.68l-1.7-1.07a6.94 6.94 0 0 0-11.07-8.28L16 7.29l-1.87-2.1A6.94 6.94 0 0 0 3.41 14l-1.6 1.19ZM16 31.18l-7.74-8.51 1.48-1.34L16 28.21l6.26-6.88 1.48 1.34z"}),(0,r.Y)("path",{d:"m16.16 23.29-4.1-7.17L10.62 19H0v-2h9.38l2.56-5.12 3.9 6.83 4.13-10.32L23.66 17H32v2h-9.66l-2.31-5.39z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_monitoring.539c0cbd.chunk.js.map