"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[1123],{87946:(e,r,t)=>{t.r(r),t.d(r,{icon:()=>a});t(65043);var n=t(77724),l=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var r=1;r<arguments.length;r++){var t=arguments[r];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},i.apply(null,arguments)}var a=function(e){var r=e.title,t=e.titleId,a=function(e,r){if(null==e)return{};var t,n,l=function(e,r){if(null==e)return{};var t={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(r.indexOf(n)>=0)continue;t[n]=e[n]}return t}(e,r);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)t=i[n],r.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(l[t]=e[t])}return l}(e,l);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":t},a),r?(0,n.Y)("title",{id:t},r):null,(0,n.Y)("circle",{cx:8,cy:8,r:4}))}}}]);
//# sourceMappingURL=icon.dot.a0097d3a.chunk.js.map