"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[96142],{37501:(e,l,n)=>{n.r(l),n.d(l,{icon:()=>a});n(65043);var t=n(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},i.apply(null,arguments)}var a=function(e){var l=e.title,n=e.titleId,a=function(e,l){if(null==e)return{};var n,t,r=function(e,l){if(null==e)return{};var n={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(l.indexOf(t)>=0)continue;n[t]=e[t]}return n}(e,l);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(t=0;t<i.length;t++)n=i[t],l.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,t.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":n},a),l?(0,t.Y)("title",{id:n},l):null,(0,t.Y)("g",{fill:"none",fillRule:"evenodd"},(0,t.Y)("path",{fill:"#119639",d:"m16 0 13.856 8v16L16 32 2.144 24V8z"}),(0,t.Y)("path",{fill:"#FFF",fillRule:"nonzero",d:"M11.17 13.512v8.376a1.607 1.607 0 1 1-3.215 0V9.632c0-1.432 1.731-2.149 2.744-1.136l9.51 9.512V9.632a1.607 1.607 0 0 1 3.215 0v12.256c0 1.432-1.731 2.149-2.744 1.136l-9.51-9.512Z"})))}}}]);
//# sourceMappingURL=icon.logo_nginx.a40f3231.chunk.js.map