"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[6418],{21937:(l,t,e)=>{e.r(t),e.d(t,{icon:()=>a});e(65043);var n=e(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(l){for(var t=1;t<arguments.length;t++){var e=arguments[t];for(var n in e)({}).hasOwnProperty.call(e,n)&&(l[n]=e[n])}return l},i.apply(null,arguments)}var a=function(l){var t=l.title,e=l.titleId,a=function(l,t){if(null==l)return{};var e,n,r=function(l,t){if(null==l)return{};var e={};for(var n in l)if({}.hasOwnProperty.call(l,n)){if(t.indexOf(n)>=0)continue;e[n]=l[n]}return e}(l,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(l);for(n=0;n<i.length;n++)e=i[n],t.indexOf(e)>=0||{}.propertyIsEnumerable.call(l,e)&&(r[e]=l[e])}return r}(l,r);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":e},a),t?(0,n.Y)("title",{id:e},t):null,(0,n.Y)("g",{fill:"none"},(0,n.Y)("path",{fill:"#FFAE00",d:"M16 32 0 11.454l6.931-10.38L16 0l9.069 1.074L32 11.454z"}),(0,n.Y)("path",{fill:"#EC6C00",d:"M16 32 0 11.454h32z"}),(0,n.Y)("path",{fill:"#FFAE00",d:"M16 32 6.477 11.454h19.045z"}),(0,n.Y)("path",{fill:"#FFEFB4",d:"M16 0 6.477 11.454h19.045z"}),(0,n.Y)("path",{fill:"#FFAE00",d:"M6.932 1.074 3.369 6.3.001 11.454h6.542zM25.069 1.074 28.632 6.3 32 11.454h-6.542z"}),(0,n.Y)("path",{fill:"#FED305",d:"m6.931 1.074-.453 10.38L16 0zM25.069 1.074l.453 10.38L16 0z"})))}}}]);
//# sourceMappingURL=icon.logo_sketch.454d4690.chunk.js.map