"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[65967],{33566:(e,a,n)=>{n.r(a),n.d(a,{icon:()=>i});n(65043);var t=n(77724),r=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var a=1;a<arguments.length;a++){var n=arguments[a];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},l.apply(null,arguments)}var i=function(e){var a=e.title,n=e.titleId,i=function(e,a){if(null==e)return{};var n,t,r=function(e,a){if(null==e)return{};var n={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(a.indexOf(t)>=0)continue;n[t]=e[t]}return n}(e,a);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(t=0;t<l.length;t++)n=l[t],a.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,t.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":n},i),a?(0,t.Y)("title",{id:n},a):null,(0,t.Y)("path",{fillRule:"evenodd",d:"M6 3H3v3h3V3ZM3 2a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h3a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H3Zm3 8H3v3h3v-3ZM3 9a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h3a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1H3Zm10-6h-3v3h3V3Zm-3-1a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h3a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1h-3Zm3 8h-3v3h3v-3Zm-3-1a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h3a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1h-3Z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.spaces.59fc3f28.chunk.js.map