"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[77475],{10722:(e,n,t)=>{t.r(n),t.d(n,{icon:()=>a});t(65043);var r=t(77724),l=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var n=1;n<arguments.length;n++){var t=arguments[n];for(var r in t)({}).hasOwnProperty.call(t,r)&&(e[r]=t[r])}return e},i.apply(null,arguments)}var a=function(e){var n=e.title,t=e.titleId,a=function(e,n){if(null==e)return{};var t,r,l=function(e,n){if(null==e)return{};var t={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(n.indexOf(r)>=0)continue;t[r]=e[r]}return t}(e,n);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(r=0;r<i.length;r++)t=i[r],n.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(l[t]=e[t])}return l}(e,l);return(0,r.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":t},a),n?(0,r.Y)("title",{id:t},n):null,(0,r.Y)("path",{fillRule:"evenodd",d:"M1 1h14v14H1V1zm13 12v1H2V2h12v11zM6 6h4v4H6V6zm-1 5V5h6v6H5z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.namespace.6c03044b.chunk.js.map