"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[21296],{97423:(e,n,t)=>{t.r(n),t.d(n,{icon:()=>i});t(65043);var r=t(77724),a=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var n=1;n<arguments.length;n++){var t=arguments[n];for(var r in t)({}).hasOwnProperty.call(t,r)&&(e[r]=t[r])}return e},l.apply(null,arguments)}var i=function(e){var n=e.title,t=e.titleId,i=function(e,n){if(null==e)return{};var t,r,a=function(e,n){if(null==e)return{};var t={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(n.indexOf(r)>=0)continue;t[r]=e[r]}return t}(e,n);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(r=0;r<l.length;r++)t=l[r],n.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(a[t]=e[t])}return a}(e,a);return(0,r.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":t},i),n?(0,r.Y)("title",{id:t},n):null,(0,r.Y)("path",{fillRule:"evenodd",d:"M6 2H1v12h3V7a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v1h4V4H7c-.621 0-1-.379-1-1V2Zm10 6v6a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.25a.75.75 0 0 1 .75.75l-.004.206C6.99 2.317 6.974 3 7 3h8a1 1 0 0 1 1 1v4Zm-1 1h-4a1 1 0 0 1-1-1V7H5v7h10V9ZM2 4.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5ZM6.5 9a.5.5 0 0 0 0 1h2a.5.5 0 0 0 0-1h-2Z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.submodule.0de0e2cd.chunk.js.map