"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[12706],{47825:(e,l,n)=>{n.r(l),n.d(l,{icon:()=>a});n(65043);var t=n(77724),i=["title","titleId"];function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}var a=function(e){var l=e.title,n=e.titleId,a=function(e,l){if(null==e)return{};var n,t,i=function(e,l){if(null==e)return{};var n={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(l.indexOf(t)>=0)continue;n[t]=e[t]}return n}(e,l);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);for(t=0;t<r.length;t++)n=r[t],l.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(i[n]=e[n])}return i}(e,i);return(0,t.Y)("svg",r({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":n},a),l?(0,t.Y)("title",{id:n},l):null,(0,t.Y)("path",{fill:"#00BFB3",fillRule:"evenodd",d:"M16 0c-2.918 0-5.645.794-8 2.158 4.777 2.768 8 7.923 8 13.842 0 5.919-3.223 11.074-8 13.842A15.907 15.907 0 0 0 16 32c8.837 0 16-7.163 16-16S24.837 0 16 0z",clipRule:"evenodd"}),(0,t.Y)("path",{fill:"#FEC514",fillRule:"evenodd",d:"M8 24h2.222A12.996 12.996 0 0 0 13 16c0-2.935-1.012-5.744-2.778-8H8a8 8 0 0 0 0 16z",clipRule:"evenodd"}),(0,t.Y)("path",{fillRule:"evenodd",d:"M16 8h-2.152A15.877 15.877 0 0 1 16 16c0 2.918-.786 5.647-2.152 8H16a8 8 0 0 0 0-16z",className:"euiIcon__fillNegative",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.logo_enterprise_search.8ed25c99.chunk.js.map