"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[23074],{62273:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>i});l(65043);var r=l(77724),n=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var r in l)({}).hasOwnProperty.call(l,r)&&(e[r]=l[r])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,l=e.titleId,i=function(e,t){if(null==e)return{};var l,r,n=function(e,t){if(null==e)return{};var l={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;l[r]=e[r]}return l}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)l=a[r],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(n[l]=e[l])}return n}(e,n);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":l},i),t?(0,r.Y)("title",{id:l},t):null,(0,r.Y)("path",{fill:"#0080D5",d:"M19.5.938a7.002 7.002 0 0 0-7 0l-8 4.619A7 7 0 0 0 1 11.62v9.237a7 7 0 0 0 3.5 6.062l7.5 4.33V17.979a7 7 0 0 1 3.5-6.062L27 5.276 19.5.939Z"}),(0,r.Y)("path",{d:"M19.5.938a7.002 7.002 0 0 0-7 0L5 5.277l11 6.35 11-6.35-7.5-4.34Z",className:"euiIcon__fillNegative"}),(0,r.Y)("path",{fill:"#FA744E",d:"m28.435 7.76-10.026 5.79a6.994 6.994 0 0 1 1.59 4.428v13.27l7.5-4.33a7 7 0 0 0 3.5-6.061v-9.238a6.992 6.992 0 0 0-1.586-4.422l-.978.564Z"}))}}}]);
//# sourceMappingURL=icon.logo_app_search.8d98ced6.chunk.js.map