"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[46068],{46195:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>i});l(65043);var r=l(77724),n=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var r in l)({}).hasOwnProperty.call(l,r)&&(e[r]=l[r])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,l=e.titleId,i=function(e,t){if(null==e)return{};var l,r,n=function(e,t){if(null==e)return{};var l={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;l[r]=e[r]}return l}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)l=a[r],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(n[l]=e[l])}return n}(e,n);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":l},i),t?(0,r.Y)("title",{id:l},t):null,(0,r.Y)("path",{fillRule:"evenodd",d:"M10 1a1 1 0 0 1 .707.293l4 4A1 1 0 0 1 15 6v5a1 1 0 0 1-.293.707l-4 4A1 1 0 0 1 10 16H5a1 1 0 0 1-.707-.293l-4-4A1 1 0 0 1 0 11V6a1 1 0 0 1 .293-.707l4-4A1 1 0 0 1 5 1h5ZM4.146 5.146a.5.5 0 0 1 .708 0L7.5 7.793l2.646-2.647a.5.5 0 0 1 .708.708L8.207 8.5l2.647 2.646a.5.5 0 0 1-.708.708L7.5 9.207l-2.646 2.647a.5.5 0 0 1-.708-.708L6.793 8.5 4.146 5.854a.5.5 0 0 1 0-.708Z"}))}}}]);
//# sourceMappingURL=icon.errorFilled.b02d7f1b.chunk.js.map