"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[94197],{66268:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>a});r(65043);var n=r(77724),l=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},i.apply(null,arguments)}var a=function(e){var t=e.title,r=e.titleId,a=function(e,t){if(null==e)return{};var r,n,l=function(e,t){if(null==e)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;r[n]=e[n]}return r}(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)r=i[n],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(l[r]=e[r])}return l}(e,l);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":r},a),t?(0,n.Y)("title",{id:r},t):null,(0,n.Y)("path",{fill:"#0080D5",d:"M15 20H4V0h11c5.522 0 10 4.478 10 10s-4.478 10-10 10"}),(0,n.Y)("path",{fill:"#00C2B3",d:"M26.702 15.624C24.6 19.979 20.152 23 15 23H4v9h15c5.522 0 10-4.478 10-10a9.952 9.952 0 0 0-2.298-6.376"}),(0,n.Y)("path",{d:"M24.338 13.554A9.942 9.942 0 0 0 19 12H4v8h11c4.27 0 7.903-2.68 9.338-6.446",className:"euiIcon__fillNegative"}))}}}]);
//# sourceMappingURL=icon.logo_beats.ab06d0b5.chunk.js.map