"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[76375],{88702:(t,e,r)=>{r.r(e),r.d(e,{icon:()=>a});r(65043);var n=r(77724),i=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var n in r)({}).hasOwnProperty.call(r,n)&&(t[n]=r[n])}return t},l.apply(null,arguments)}var a=function(t){var e=t.title,r=t.titleId,a=function(t,e){if(null==t)return{};var r,n,i=function(t,e){if(null==t)return{};var r={};for(var n in t)if({}.hasOwnProperty.call(t,n)){if(e.indexOf(n)>=0)continue;r[n]=t[n]}return r}(t,e);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(t);for(n=0;n<l.length;n++)r=l[n],e.indexOf(r)>=0||{}.propertyIsEnumerable.call(t,r)&&(i[r]=t[r])}return i}(t,i);return(0,n.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},a),e?(0,n.Y)("title",{id:r},e):null,(0,n.Y)("rect",{width:12,height:12,x:2,y:2,rx:3}),(0,n.Y)("rect",{width:11,height:11,x:2.5,y:2.5,className:"euiSwatchInput__stroke",rx:2}))}}}]);
//# sourceMappingURL=icon.swatch_input.79d9b582.chunk.js.map