"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[71701],{53500:(e,t,n)=>{n.r(t),n.d(t,{icon:()=>i});n(65043);var a=n(77724),r=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var a in n)({}).hasOwnProperty.call(n,a)&&(e[a]=n[a])}return e},l.apply(null,arguments)}var i=function(e){var t=e.title,n=e.titleId,i=function(e,t){if(null==e)return{};var n,a,r=function(e,t){if(null==e)return{};var n={};for(var a in e)if({}.hasOwnProperty.call(e,a)){if(t.indexOf(a)>=0)continue;n[a]=e[a]}return n}(e,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(a=0;a<l.length;a++)n=l[a],t.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,a.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":n},i),t?(0,a.Y)("title",{id:n},t):null,(0,a.Y)("path",{d:"M4 6h22v7h-2V8H6v19H4V6Z",className:"euiIcon__fillSecondary"}),(0,a.Y)("path",{d:"M10 12H7v-2h3v2Zm1 0h12v-2H11v2Z",className:"euiIcon__fillSecondary"}),(0,a.Y)("path",{d:"M24 1H0v26h1l-1 .001v.026a2.256 2.256 0 0 0 .01.185 4.318 4.318 0 0 0 .563 1.792C1.194 30.064 2.399 31 4.5 31H17v-2H8.429a4.32 4.32 0 0 0 .57-1.927v-.062L9 27.005v-.003S9 27 8 27h1V16h21v3h2v-5H7v12.995l-.003.05a2.32 2.32 0 0 1-.298.95C6.442 28.436 5.898 29 4.5 29c-1.398 0-1.942-.564-2.199-1.004a2.32 2.32 0 0 1-.3-1.001L2 26.992V3h20v3h2V1Z"}),(0,a.Y)("path",{d:"M23 21v3h-3v2h3v3h2v-3h3v-2h-3v-3h-2Z"}),(0,a.Y)("path",{fillRule:"evenodd",d:"M24 32a7 7 0 1 0 0-14 7 7 0 0 0 0 14Zm0-2a5 5 0 1 0 0-10 5 5 0 0 0 0 10Z"}))}}}]);
//# sourceMappingURL=icon.app_cases.e1e2fe7e.chunk.js.map