"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[81855],{49774:(e,l,n)=>{n.r(l),n.d(l,{icon:()=>a});n(65043);var t=n(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},i.apply(null,arguments)}var a=function(e){var l=e.title,n=e.titleId,a=function(e,l){if(null==e)return{};var n,t,r=function(e,l){if(null==e)return{};var n={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(l.indexOf(t)>=0)continue;n[t]=e[t]}return n}(e,l);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(t=0;t<i.length;t++)n=i[t],l.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,t.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":n},a),l?(0,t.Y)("title",{id:n},l):null,(0,t.Y)("path",{fillRule:"evenodd",d:"M14 5.151v6.269l-5.5 3.142V8.29L14 5.151zm-.508-.861L8 1.152 2.508 4.29 8 7.424l5.492-3.134zM2 5.151l5.5 3.14v6.272L2 11.42V5.15zM15 12l-7 4-7-4V4l7-4 7 4v8z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.kubernetesPod.f2ec4a06.chunk.js.map