"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[73627],{43330:(e,n,r)=>{r.r(n),r.d(n,{icon:()=>i});r(65043);var t=r(77724),l=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var n=1;n<arguments.length;n++){var r=arguments[n];for(var t in r)({}).hasOwnProperty.call(r,t)&&(e[t]=r[t])}return e},a.apply(null,arguments)}var i=function(e){var n=e.title,r=e.titleId,i=function(e,n){if(null==e)return{};var r,t,l=function(e,n){if(null==e)return{};var r={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(n.indexOf(t)>=0)continue;r[t]=e[t]}return r}(e,n);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(t=0;t<a.length;t++)r=a[t],n.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(l[r]=e[r])}return l}(e,l);return(0,t.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 16 16","aria-labelledby":r},i),n?(0,t.Y)("title",{id:r},n):null,(0,t.Y)("path",{fillRule:"evenodd",d:"M0 2.5A.5.5 0 0 1 .5 2h15a.5.5 0 0 1 0 1H15v10h.5a.5.5 0 0 1 0 1H.5a.5.5 0 0 1 0-1H1V3H.5a.5.5 0 0 1-.5-.5zM2 3h12v10H2V3zm3 2a.5.5 0 0 0-.5.5v5a.5.5 0 0 0 1 0v-5A.5.5 0 0 0 5 5zm5.5.5a.5.5 0 0 1 1 0v5a.5.5 0 0 1-1 0v-5zM8 5a.5.5 0 0 0-.5.5v5a.5.5 0 0 0 1 0v-5A.5.5 0 0 0 8 5z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.container.63756e89.chunk.js.map