"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[23230],{42101:(e,t,a)=>{a.r(t),a.d(t,{icon:()=>i});a(65043);var n=a(77724),r=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var a=arguments[t];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},l.apply(null,arguments)}var i=function(e){var t=e.title,a=e.titleId,i=function(e,t){if(null==e)return{};var a,n,r=function(e,t){if(null==e)return{};var a={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;a[n]=e[n]}return a}(e,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(n=0;n<l.length;n++)a=l[n],t.indexOf(a)>=0||{}.propertyIsEnumerable.call(e,a)&&(r[a]=e[a])}return r}(e,r);return(0,n.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":a},i),t?(0,n.Y)("title",{id:a},t):null,(0,n.Y)("path",{fillRule:"evenodd",d:"M2 2a1 1 0 0 0-1 1v7a1 1 0 0 0 1 1h4l-1 3H3.5a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1H11l-1-3h4a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2zm0 8V3h12v7H2zm5 1h2l1 3H6l1-3z",clipRule:"evenodd"}),(0,n.Y)("path",{d:"M4.5 6a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm3 0a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm2.5.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5z"}))}}}]);
//# sourceMappingURL=icon.desktop.4728299d.chunk.js.map