"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[71632],{16503:(e,a,t)=>{t.r(a),t.d(a,{icon:()=>i});t(65043);var n=t(77724),r=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var a=1;a<arguments.length;a++){var t=arguments[a];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},l.apply(null,arguments)}var i=function(e){var a=e.title,t=e.titleId,i=function(e,a){if(null==e)return{};var t,n,r=function(e,a){if(null==e)return{};var t={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(a.indexOf(n)>=0)continue;t[n]=e[n]}return t}(e,a);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(n=0;n<l.length;n++)t=l[n],a.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(r[t]=e[t])}return r}(e,r);return(0,n.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":t},i),a?(0,n.Y)("title",{id:t},a):null,(0,n.Y)("path",{d:"M.586 2.586A2 2 0 0 0 0 4h1a1 1 0 0 1 1-1V2a2 2 0 0 0-1.414.586ZM2 2h10.5a.5.5 0 0 1 0 1H2V2ZM0 4h1v6.5a.5.5 0 0 1-1 0V4Zm2.586.586A2 2 0 0 0 2 6h1a1 1 0 0 1 1-1V4a2 2 0 0 0-1.414.586Zm0 8.828A2 2 0 0 1 2 12h1a1 1 0 0 0 1 1v1a2 2 0 0 1-1.414-.586Zm12.828-8.828A2 2 0 0 1 16 6h-1a1 1 0 0 0-1-1V4a2 2 0 0 1 1.414.586Zm0 8.828A2 2 0 0 0 16 12h-1a1 1 0 0 1-1 1v1a2 2 0 0 0 1.414-.586ZM4 4h10v1H4zM3 7h12v1H3zm1 6h10v1H4zM2 6h1v6H2zm13 0h1v6h-1zm-5.5 4a.5.5 0 0 1 0 1H7.496a.5.5 0 0 1 0-1H9.5Zm4 0a.5.5 0 0 1 0 1h-2.004a.5.5 0 0 1 0-1H13.5Z"}))}}}]);
//# sourceMappingURL=icon.payment.b67de02e.chunk.js.map