"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[33895],{77726:(e,t,n)=>{n.r(t),n.d(t,{icon:()=>i});n(65043);var r=n(77724),l=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)({}).hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,n=e.titleId,i=function(e,t){if(null==e)return{};var n,r,l=function(e,t){if(null==e)return{};var n={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)n=a[r],t.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(l[n]=e[n])}return l}(e,l);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":n},i),t?(0,r.Y)("title",{id:n},t):null,(0,r.Y)("path",{d:"M4 2.75a.75.75 0 0 1 1.5 0v10.5a.75.75 0 0 1-1.5 0V2.75Z"}),(0,r.Y)("path",{fillRule:"evenodd",d:"M13.135 14.043a.75.75 0 0 0 .025-1.06l-4.59-4.81a.25.25 0 0 1 0-.346l4.59-4.81a.75.75 0 0 0-1.085-1.035l-4.59 4.81a1.75 1.75 0 0 0 0 2.416l4.59 4.81c.286.3.76.31 1.06.024Z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.arrowStart.62c5a8a5.chunk.js.map