"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[12373],{22972:(t,e,r)=>{r.r(e),r.d(e,{icon:()=>a});r(65043);var n=r(77724),l=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var n in r)({}).hasOwnProperty.call(r,n)&&(t[n]=r[n])}return t},i.apply(null,arguments)}var a=function(t){var e=t.title,r=t.titleId,a=function(t,e){if(null==t)return{};var r,n,l=function(t,e){if(null==t)return{};var r={};for(var n in t)if({}.hasOwnProperty.call(t,n)){if(e.indexOf(n)>=0)continue;r[n]=t[n]}return r}(t,e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(t);for(n=0;n<i.length;n++)r=i[n],e.indexOf(r)>=0||{}.propertyIsEnumerable.call(t,r)&&(l[r]=t[r])}return l}(t,l);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},a),e?(0,n.Y)("title",{id:r},e):null,(0,n.Y)("path",{d:"M10 13.999v-2.99h1V15H0V1h11v3.999h-1V2.001H1v11.998z"}),(0,n.Y)("path",{d:"M4.5 10V9H11v1zM13 9.5l3-3-3-3z"}),(0,n.Y)("path",{d:"M5.5 6.5v6l-3-3zM7 7V6h6.5v1z"}))}}}]);
//# sourceMappingURL=icon.inputOutput.28ed931d.chunk.js.map